@@ -20,9 +20,8 @@ You should have received a copy of the GNU General Public License
 along with APAV.  If not, see <http://www.gnu.org/licenses/>.
 """
 
-from __future__ import annotations
 from typing import Sequence, Tuple, List, Dict, Any, Union, Type, Optional, TYPE_CHECKING
-from numbers import Real
+from numbers import Real, Number
 from numpy import ndarray
 from lmfit.models import Model
 from periodictable.core import Element
