@@ -20,23 +20,25 @@ You should have received a copy of the GNU General Public License
 along with APAV.  If not, see <http://www.gnu.org/licenses/>.
 """
 
-from __future__ import annotations
-
 import os
 
 import numpy as n
 import pyqtgraph as pg
 from copy import copy
 
-from apav.vtkall import *
+# from apav.vtkall import *
 from apav.qt import *
-from apav.visualization.base import BaseVisualization#, VTKVisualization
+from apav.visualization.base import BaseVisualization
 from apav.qtwidgets.massrangewidget import QMassRangeWidget
 from apav.core import histogram
-from apav.visualization.vtkhelpers import point_cloud, point_dataset
 from apav.utils.helpers import modifying
 import apav.qtwidgets.controls as controls
 
+from typing import TYPE_CHECKING
+
+if TYPE_CHECKING:
+    from apav.analysis.massspectrum import LocalBkgCorrectedMassSpectrum, RangedMassSpectrum, NoiseCorrectedMassSpectrum
+
 pg.setConfigOption("foreground", "k")
 pg.setConfigOption("background", "w")
 pg.setConfigOption("antialias", True)
@@ -46,6 +48,7 @@ class Plotter1D(pg.PlotWidget):
     """
     Base 1D plotter settings
     """
+
     def __init__(self, parent: QWidget, xlabel: str, ylabel: str, xunits: str = "", yunits: str = ""):
         super().__init__(parent)
         self.getPlotItem().showAxis("right")
@@ -79,8 +82,9 @@ class MassSpectrumPlot(BaseVisualization):
 
         self.plot_layout.addWidget(self.plotter)
 
-        self.prox = pg.SignalProxy(self.plotter.getPlotItem().scene().sigMouseMoved, rateLimit=30,
-                                   slot=self.slotOnMouseMoved)
+        self.prox = pg.SignalProxy(
+            self.plotter.getPlotItem().scene().sigMouseMoved, rateLimit=30, slot=self.slotOnMouseMoved
+        )
 
         self.data = None
         self.makeToolBar()
@@ -154,11 +158,13 @@ class MassSpectrumPlot(BaseVisualization):
             self.plotter.getPlotItem().clear()
             return
 
-        self.data = self.ref_data.mass_histogram(bin_width=bin_width,
-                                                 multiplicity=mult,
-                                                 norm=self.norm.isChecked(),
-                                                 lower=self.lower.value(),
-                                                 upper=self.upper.value())
+        self.data = self.ref_data.mass_histogram(
+            bin_width=bin_width,
+            multiplicity=mult,
+            norm=self.norm.isChecked(),
+            lower=self.lower.value(),
+            upper=self.upper.value(),
+        )
 
         # x = self.data[0] - bin_width/2
         # x = n.hstack((x, x[-1] + bin_width))
@@ -168,7 +174,7 @@ class MassSpectrumPlot(BaseVisualization):
         pos = self.plotter.getPlotItem().getViewBox().mapSceneToView(event[0])
 
         x, y = pos.x(), pos.y()
-        idx = int((x - self.lower.value())/self.bin_width.value())
+        idx = int((x - self.lower.value()) / self.bin_width.value())
         try:
             y = self.data[1][idx]
         except IndexError:
@@ -189,7 +195,7 @@ class MassSpectrumPlot(BaseVisualization):
 
 
 class MassSpectrumPlotRanged(BaseVisualization):
-    def __init__(self, ranged_mass_spec):
+    def __init__(self, ranged_mass_spec: "RangedMassSpectrum"):
         self.rmass = ranged_mass_spec
         super().__init__(ranged_mass_spec.roi)
         self.resize(1200, 600)
@@ -209,8 +215,9 @@ class MassSpectrumPlotRanged(BaseVisualization):
 
         self.plot_layout.addWidget(self.plotter)
 
-        self.prox = pg.SignalProxy(self.plotter.getPlotItem().scene().sigMouseMoved, rateLimit=30,
-                                   slot=self.slotOnMouseMoved)
+        self.prox = pg.SignalProxy(
+            self.plotter.getPlotItem().scene().sigMouseMoved, rateLimit=30, slot=self.slotOnMouseMoved
+        )
 
         self.makeToolBar()
 
@@ -221,12 +228,17 @@ class MassSpectrumPlotRanged(BaseVisualization):
             idx = n.argwhere((self.data[0] >= rng.lower) & (self.data[0] <= rng.upper))[:, 0]
             rngx = histogram.centers2edges(self.data[0][idx], self.rmass.bin_width)
             fill = QBrush(QColor.fromRgbF(*rng.color))
-            self.plotter.getPlotItem().plot(rngx, self.data[1][idx], stepMode="center", brush=fill, fillLevel=0, pen=None)
+            line = pg.mkPen(None)
+            self.plotter.getPlotItem().plot(
+                rngx, self.data[1][idx], stepMode="center", brush=fill, fillLevel=0, pen=line
+            )
 
         # Draw line spectrum
         self.centers = histogram.centers2edges(self.data[0], self.rmass.bin_width)
         line = pg.mkPen(color=(0, 0, 0), width=1)
-        self.mass_spectrum_item = self.plotter.getPlotItem().plot(self.centers, self.data[1], stepMode="center", pen=line)
+        self.mass_spectrum_item = self.plotter.getPlotItem().plot(
+            self.centers, self.data[1], stepMode="center", pen=line
+        )
 
         # Range viewer
         range_dock = QDockWidget("Mass ranges", self)
@@ -249,10 +261,10 @@ class MassSpectrumPlotRanged(BaseVisualization):
         ystride = ymax - ymin
 
         # Pad the x, y extents
-        xmin -= 0.1*xstride
-        xmax += 0.1*xstride
-        ymin -= 0.1*ystride
-        ymax += 0.1*ystride
+        xmin -= 0.1 * xstride
+        xmax += 0.1 * xstride
+        ymin -= 0.1 * ystride
+        ymax += 0.1 * ystride
 
         # Don't let ymin be zero in case the plot is int log scale
         if ymin < 1:
@@ -306,7 +318,7 @@ class MassSpectrumPlotRanged(BaseVisualization):
         pos = self.plotter.getPlotItem().getViewBox().mapSceneToView(event[0])
 
         x, y = pos.x(), pos.y()
-        idx = int(x/self.rmass.bin_width)
+        idx = int(x / self.rmass.bin_width)
         try:
             counts = self.data[1][idx]
         except IndexError:
@@ -344,39 +356,97 @@ class MassSpectrumPlotNoiseCorrected(MassSpectrumPlotRanged):
 
         self.setWindowTitle("Noise corrected mass histogram - {}".format(os.path.basename(self.ref_data.filepath)))
 
-        line = pg.mkPen(color=(255, 0, 255), width=1)
+        line = pg.mkPen(color=(255, 0, 255), width=2)
         bkg_min = self.nmass.noise_background.lower
         x = self.nmass.noise_fit_data[0]
-        idx = n.argmin(n.abs(x-bkg_min))
-        # print(bkg_min, idx)
-        self.noise_fit_item = self.plotter.getPlotItem().plot(self.nmass.noise_fit_data[0][idx:],
-                                                              self.nmass.noise_fit_data[1][idx:],
-                                                              pen=line)
+        idx = n.argmin(n.abs(x - bkg_min))
+        self.noise_fit_item = self.plotter.getPlotItem().plot(
+            self.nmass.noise_fit_data[0][idx:], self.nmass.noise_fit_data[1][idx:], pen=line
+        )
 
         self.legend = pg.LegendItem(offset=(-20, 10))
         self.legend.setParentItem(self.plotter.getPlotItem())
         self.legend.addItem(self.mass_spectrum_item, "Uncorrected mass spectrum")
-        self.legend.addItem(self.noise_fit_item, f"Background noise fit")# ({self.nmass.noise_background.eval_func.descr})")
+        self.legend.addItem(self.noise_fit_item, f"Noise background")
 
 
 class MassSpectrumPlotLocalBkgCorrected(MassSpectrumPlotNoiseCorrected):
-    def __init__(self, local_bkg_mass):
+    def __init__(self, local_bkg_mass: "LocalBkgCorrectedMassSpectrum"):
         super().__init__(local_bkg_mass)
         self.lmass = local_bkg_mass
-        self.setWindowTitle("Local background corrected mass histogram - {}".format(os.path.basename(self.ref_data.filepath)))
+        self.setWindowTitle(
+            "Local background corrected mass histogram - {}".format(os.path.basename(self.ref_data.filepath))
+        )
+
+        fit_pen = pg.mkPen("g", width=3)
+        inc_pen = pg.mkPen("r", width=2, style=Qt.DotLine)
+        corr_pen = pg.mkPen("r", width=3)
+        bkg_pen = pg.mkPen(0.0, width=4)
+
+        def correct_edges(ary: n.ndarray):
+            """
+            The fits are inherently evaluated on with end points at the center of the bins
+            we need the edges in order to visualize on the histogram
+            :param ary: array histogram centers
+            """
+            dx = (ary[1] - ary[0]) / 2
+            return n.concatenate([[ary[0] - dx], ary, [ary[-1] + dx]])
+
+        # We only want to add items to legend one time, keep track if it has been added
+        legend_fit_added = False
+        legend_bkg_added = False
 
         for bkg in self.lmass.background_collection:
-            lower_idx = int(min(i[0] for i in bkg.fit_intervals)/self.lmass.bin_width)
-            # upper_idx = int(max(i[1] for i in bkg.include_intervals)/self.lmass.bin_width)
-            x = self.lmass.histogram[0][lower_idx:]
-            noise_y = self.lmass.noise_background.eval(x)
-            sig_y = bkg.eval(x)
-            self.plotter.getPlotItem().plot(x, sig_y + noise_y, pen=(0, 0, 0))
+            # Plot line for fit interval
+            lower_fit_idx = round(min(i[0] for i in bkg.fit_intervals) / self.lmass.bin_width)
+            upper_fit_idx = round(max(i[1] for i in bkg.fit_intervals) / self.lmass.bin_width)
+            x_fit = correct_edges(self.lmass.histogram[0][lower_fit_idx:upper_fit_idx])
+            sig_y_fit = bkg.eval(x_fit)
+            noise_y_fit = self.lmass.noise_background.eval(x_fit)
+            y_fit = sig_y_fit + noise_y_fit
+
+            self.plotter.getPlotItem().plot(x_fit, y_fit, pen=bkg_pen)
+            item = self.plotter.getPlotItem().plot(x_fit, y_fit, pen=fit_pen)
+
+            if legend_fit_added is False:
+                self.legend.addItem(item, f"Local background fit")
+                legend_fit_added = True
+
+            bkg_max_range_val = None
+
+            for rng in self.lmass.ranges:
+                if not bkg.contains_range(rng):
+                    continue
+
+                lower_inc_idx = round(rng.lower / self.lmass.bin_width)
+                upper_inc_idx = round(rng.upper / self.lmass.bin_width)
+                x_inc = correct_edges(self.lmass.histogram[0][lower_inc_idx:upper_inc_idx])
+                noise_y_inc = self.lmass.noise_background.eval(x_inc)
+                sig_y_inc = bkg.eval(x_inc)
+                y_inc = sig_y_inc + noise_y_inc
+
+                self.plotter.getPlotItem().plot(x_inc, y_inc, pen=bkg_pen)
+                item = self.plotter.getPlotItem().plot(x_inc, y_inc, pen=corr_pen)
+
+                if legend_bkg_added is False:
+                    self.legend.addItem(item, f"Local background correction")
+                    legend_bkg_added = True
+
+                bkg_max_range_val = int(rng.upper / self.lmass.bin_width)
+
+            # Plot line from fit interval to max included Range
+            if bkg_max_range_val is not None:
+                x_corr = correct_edges(self.lmass.histogram[0][upper_fit_idx:bkg_max_range_val])
+                sig_y_corr = bkg.eval(x_corr)
+                noise_y_corr = self.lmass.noise_background.eval(x_corr)
+                y_corr = sig_y_corr + noise_y_corr
+
+                self.plotter.getPlotItem().plot(x_corr, y_corr, pen=inc_pen)
 
 
-class InteractiveCorrelationHistogramPlot(BaseVisualization):
+class CorrelationHistogramPlot(BaseVisualization):
     def __init__(self, corr_hist):
-        super().__init__(copy(corr_hist))
+        super().__init__(corr_hist)
         self.resize(700, 500)
         self.setWindowTitle("Correlation Histogram - {}".format(os.path.basename(corr_hist.roi.filepath)))
         self.setMouseTracking(True)
@@ -416,8 +486,8 @@ class InteractiveCorrelationHistogramPlot(BaseVisualization):
         ysize = self.data.shape[1]
 
         x, y = pos.x(), pos.y()
-        idx = int((x - xrng[0])/bin)
-        idy = int((y - yrng[0])/bin)
+        idx = int((x - xrng[0]) / bin)
+        idy = int((y - yrng[0]) / bin)
         try:
             z = self.data[idx, idy]
         except:
@@ -556,8 +626,10 @@ class InteractiveCorrelationHistogramPlot(BaseVisualization):
                 multiplicity = self.multiplicity.formattedValue()
 
                 ref_data.multiplicity = multiplicity
-                ref_data.extents = ((self.lower1.value(), self.upper1.value()),
-                                    (self.lower2.value(), self.upper2.value()))
+                ref_data.extents = (
+                    (self.lower1.value(), self.upper1.value()),
+                    (self.lower2.value(), self.upper2.value()),
+                )
                 ref_data.flip = self.flip.isChecked()
                 ref_data.symmetric = self.symmetric.isChecked()
                 ref_data.bin_width = self.binwidth.value()
@@ -602,8 +674,11 @@ class InteractiveCorrelationHistogramPlot(BaseVisualization):
         self.plotter.addItem(self.img)
 
         self.img.setImage(plot_data)
-        self.img.translate(xrng[0], yrng[0])
-        self.img.scale(binw, binw)
+        tr = self.img.transform()
+        tr.translate(xrng[0], yrng[0])
+        tr.scale(binw, binw)
+        self.img.setTransform(tr)
+
         if self.colorbar is None:
             self.colorbar = pg.HistogramLUTWidget(self, self.img)
         else:
@@ -611,9 +686,9 @@ class InteractiveCorrelationHistogramPlot(BaseVisualization):
         self.plot_layout.insertWidget(0, self.plotter)
         self.plot_layout.addWidget(self.colorbar)
 
-        self.prox = pg.SignalProxy(self.plotter.getPlotItem().scene().sigMouseMoved,
-                                   rateLimit=30,
-                                   slot=self.slotOnMouseMoved)
+        self.prox = pg.SignalProxy(
+            self.plotter.getPlotItem().scene().sigMouseMoved, rateLimit=30, slot=self.slotOnMouseMoved
+        )
 
 
 # class DetectorDeadZonePlot(VTKVisualization):
@@ -685,7 +760,3 @@ class InteractiveCorrelationHistogramPlot(BaseVisualization):
 #         # b.SetInputConnection(chart)
 #         # a = vtkOpenGLContextActor()
 #         # self.viewport.register_actors(chart)
-
-
-
-
